@@ -2,16 +2,14 @@ package JSON;
 
 use strict;
 use base qw(Exporter);
-use JSON::Parser;
-use JSON::Converter;
 
 @JSON::EXPORT = qw(objToJson jsonToObj);
 
 use vars qw($AUTOCONVERT $VERSION $UnMapping $BareKey $QuotApos
             $ExecCoderef $SkipInvalid $Pretty $Indent $Delimiter
-            $KeySort $ConvBlessed $SelfConvert);
+            $KeySort $ConvBlessed $SelfConvert $UTF8 $SingleQuote);
 
-$VERSION     = '1.03';
+$VERSION = '1.04';
 
 $AUTOCONVERT = 1;
 $SkipInvalid = 0;
@@ -23,10 +21,23 @@ $UnMapping   = 0; #
 $BareKey     = 0; # 
 $QuotApos    = 0; # 
 $KeySort     = undef; # Code-ref to provide sort ordering in converter
+$UTF8        = 0;
+$SingleQuote = 0;
+
+my $USE_UTF8;
+
+BEGIN {
+    $USE_UTF8 = $] >= 5.008 ? 1 : 0;
+    sub USE_UTF8 {  $USE_UTF8; }
+}
+
+use JSON::Parser;
+use JSON::Converter;
 
 my $parser; # JSON => Perl
 my $conv;   # Perl => JSON
 
+
 ##############################################################################
 # CONSTRCUTOR - JSON objects delegate all processes
 #                   to JSON::Converter and JSON::Parser.
@@ -48,10 +59,13 @@ sub new {
         keysort     => $KeySort    ,
         convblessed => $ConvBlessed,
         selfconvert => $SelfConvert,
+        singlequote => $SingleQuote,
         # below fields are for JSON::Parser
         unmapping   => $UnMapping,
         quotapos    => $QuotApos ,
         barekey     => $BareKey  ,
+        # common options
+        utf8        => $UTF8     ,
         # overwrite
         %opt,
     }, $class;
@@ -62,12 +76,16 @@ sub new {
 # METHODS
 ##############################################################################
 
+*parse_json = \&jsonToObj;
+
+*to_json    = \&objToJson;
+
 sub jsonToObj {
     my $self = shift;
     my $js   = shift;
 
     if(!ref($self)){ # class method
-        my $opt = __PACKAGE__->_getParamsForParser($_[0]);
+        my $opt = __PACKAGE__->_getParamsForParser($js);
         $js = $self;
         $parser ||= new JSON::Parser;
         $parser->jsonToObj($js, $opt);
@@ -107,15 +125,17 @@ sub _getParamsForParser {
     my $params;
 
     if(ref($self)){ # instance
-        my @names = qw(unmapping quotapos barekey);
-        my ($unmapping, $quotapos, $barekey) = @{$self}{ @names };
+        my @names = qw(unmapping quotapos barekey utf8);
+        my ($unmapping, $quotapos, $barekey, $utf8) = @{$self}{ @names };
         $params = {
-            unmapping => $unmapping, quotapos => $quotapos, barekey => $barekey,
+            unmapping => $unmapping, quotapos => $quotapos,
+            barekey   => $barekey,   utf8     => $utf8,
         };
     }
     else{ # class
         $params = {
-            unmapping => $UnMapping, barekey => $BareKey, quotapos => $QuotApos,
+            unmapping => $UnMapping, barekey => $BareKey,
+            quotapos  => $QuotApos,  utf8    => $UTF8,
         };
     }
 
@@ -134,21 +154,26 @@ sub _getParamsForConverter {
     my $params;
 
     if(ref($self)){ # instance
-        my @names = qw(pretty indent delimiter autoconv keysort convblessed selfconvert);
-        my ($pretty, $indent, $delimiter, $autoconv, $keysort, $convblessed, $selfconvert)
+        my @names
+         = qw(pretty indent delimiter autoconv keysort convblessed selfconvert utf8 singlequote);
+        my ($pretty, $indent, $delimiter, $autoconv,
+                $keysort, $convblessed, $selfconvert, $utf8, $singlequote)
                                                            = @{$self}{ @names };
         $params = {
-            pretty => $pretty, indent => $indent,
-            delimiter => $delimiter, autoconv => $autoconv,
-            keysort => $keysort, convblessed => $convblessed,
-            selfconvert => $selfconvert,
+            pretty      => $pretty,       indent      => $indent,
+            delimiter   => $delimiter,    autoconv    => $autoconv,
+            keysort     => $keysort,      convblessed => $convblessed,
+            selfconvert => $selfconvert,  utf8        => $utf8,
+            singlequote => $singlequote,
         };
     }
     else{ # class
         $params = {
-            pretty => $Pretty, indent => $Indent, delimiter => $Delimiter,
-            keysort => $KeySort, convblessed => $ConvBlessed,
-            selfconvert => $SelfConvert,
+            pretty      => $Pretty,       indent      => $Indent,
+            delimiter   => $Delimiter,    autoconv    => $AUTOCONVERT,
+            keysort     => $KeySort,      convblessed => $ConvBlessed,
+            selfconvert => $SelfConvert,  utf8        => $UTF8,
+            singlequote => $SingleQuote, 
         };
     }
 
@@ -164,22 +189,15 @@ sub _getParamsForConverter {
 ##############################################################################
 # ACCESSOR
 ##############################################################################
-
-sub autoconv { $_[0]->{autoconv} = $_[1] if(defined $_[1]); $_[0]->{autoconv} }
-
-sub pretty { $_[0]->{pretty} = $_[1] if(defined $_[1]); $_[0]->{pretty} }
-
-sub indent { $_[0]->{indent} = $_[1] if(defined $_[1]); $_[0]->{indent} }
-
-sub delimiter { $_[0]->{delimiter} = $_[1] if(defined $_[1]); $_[0]->{delimiter} }
-
-sub unmapping { $_[0]->{unmapping} = $_[1] if(defined $_[1]); $_[0]->{unmapping} }
-
-sub keysort { $_[0]->{keysort} = $_[1] if(defined $_[1]); $_[0]->{keysort} }
-
-sub convblessed { $_[0]->{convblessed} = $_[1] if(defined $_[1]); $_[0]->{convblessed} }
-
-sub selfconvert { $_[0]->{selfconvert} = $_[1] if(defined $_[1]); $_[0]->{selfconvert} }
+BEGIN{
+    for my $name (qw/autoconv pretty indent delimiter 
+                  unmapping keysort convblessed selfconvert singlequote/)
+    {
+        eval qq{
+            sub $name { \$_[0]->{$name} = \$_[1] if(defined \$_[1]); \$_[0]->{$name} }
+        };
+    }
+}
 
 ##############################################################################
 # NON STRING DATA
@@ -192,7 +210,7 @@ sub Number {
 
     return undef if(!defined $num);
 
-    if($num =~ /^-?(?:0|[1-9][\d]*)(?:\.[\d]*)?$/
+    if($num =~ /^-?(?:\d+)(?:\.\d*)?(?:[eE][-+]?\d+)?$/
                or $num =~ /^0[xX](?:[0-9a-zA-Z])+$/)
     {
         return bless {value => $num}, 'JSON::NotString';
@@ -581,6 +599,20 @@ the module will test for a C<toJson()> method on the object,
 and will rely on this method to obtain the converted value of
 the object.
 
+=head1 UTF8
+
+You can set a true value into $JSON::UTF8 for JSON::Parser
+and JSON::Converter to set UTF8 flag into strings contain utf8.
+
+
+=head1 CONVERT WITH SINGLE QUOTES
+
+You can set a true value into $JSON::SingleQuote for JSON::Converter
+to quote any keys and values with single quotations.
+
+You want to parse single quoted JSON data, See L</SINGLE QUOTATION OPTION>.
+
+
 =head1 EXPORT
 
 C<objToJson>, C<jsonToObj>.
@@ -589,10 +621,15 @@ C<objToJson>, C<jsonToObj>.
 
 Which name is more desirable? JSONRPC or JSON::RPC.
 
+SingleQuote and QuotApos...
+
+
 =head1 SEE ALSO
 
 L<http://www.crockford.com/JSON/>, L<JSON::Parser>, L<JSON::Converter>
 
+If you want the speed and the saving of memory usage,
+check L<JSON::Syck>.
 
 =head1 ACKNOWLEDGEMENTS
 
